# Amethyst - An experimental Minecraft server written in Elixir.
# Copyright (C) 2024 KodiCraft
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

defmodule Amethyst.ConnectionState do
  @moduledoc """
  This behaviour defines the behaviour that all connection states must implement.
  """

  alias Amethyst.ConnectionHandler
  @type t :: module()

  @callback serialize(packet :: %{packet_type: atom()}, state :: ConnectionHandler.t()) :: binary()
  @spec serialize(state :: ConnectionHandler.t(), packet :: %{packet_type: atom()}) :: binary()
  def serialize(state, packet) do
    state.connection_state.serialize(packet, state)
  end

  @callback deserialize(id :: non_neg_integer(), data :: binary(), state :: ConnectionHandler.t()) :: %{packet_type: atom()}
  @spec deserialize(state :: ConnectionHandler.t(), id :: pos_integer(), data :: binary()) :: %{packet_type: atom()}
  def deserialize(state, id, data) do
    state.connection_state.serialize(id, data, state)
  end

  @callback handle(packet :: %{packet_type: atom()}, state :: ConnectionHandler.t()) :: [ConnectionHandler.operation()]
  @spec handle(state :: ConnectionHandler.t(), packet :: %{packet_type: atom()}) :: [ConnectionHandler.operation()]
  def handle(state, packet) do
    state.connection_state.handle(packet)
  end

  @callback disconnect(reason :: String.t(), state :: ConnectionHandler.t()) :: %{packet_type: atom()}
  @spec disconnect(state :: ConnectionHandler.t(), reason :: String.t()) :: %{packet_type: atom()}
  def disconnect(state, reason) do
    state.connection_state.disconnect(reason)
  end
end
